# Project Reality Mini Mod v 0.25 Updated Feb 18, 2006
# Custom Map Support Added for:
# Strike at Karkand 2: 16, 32, 64
# Wake Island 2007: 64 (only applicable mode for this map)
# The Oasis: 16, 32, 64
# El Alamein XXL: 16, 32, 64
# CP Abadan: 16, 32, 64
# HeliWar: 16, 32, 64
# Steel Thunder: 16, 32, 64
# Jungle Fever: 16, 32, 64
# Hereford Training: 16, 32, 64
# Road to Kyongan Ni: 16, 32, 64
# Insurgency On Alcatraz Island: 16, 32, 64
# 
# PRMM v 0.25 Note: Defaults values for CP Balance set to FALSE, Commander set to FALSE, AAS Radii Scaling set to DISABLED
#
# landot AAS mod for BF2 conquest mode
#
# Version 1.7
#
# The following copyright only applies to pieces of code added or changed
# in this file compared to the original gpm_cq.py included with BF2 1.02.
# Any other code is copyright of the copyright owner of that particular code.
#
# Copyright (c) 2005, Jarkko Santala <santala@landot.org>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
# OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
# WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Contributors:
#
# Letranger, Men Dake, Nordstern, Dark Angel
#
# Thanks to everyone for great help in testing, flag order and new ideas!
#
# Please check SETTINGS below.
#

TAKEOVERTYPE_CAPTURE = 1
TAKEOVERTYPE_NEUTRALIZE = 2

SCORE_CAPTURE = 2
SCORE_NEUTRALIZE = 2
SCORE_CAPTUREASSIST = 1
SCORE_NEUTRALIZEASSIST = 1
SCORE_DEFEND = 1

Top = 0
Middle = 1
Bottom = 2

import host
import bf2
import math
from game.scoringCommon import addScore, RPL
from bf2 import g_debug

g_controlPoints = [] # cache, as this map won't change

global aasTip
global aasLast
global aasNextCp
global aasRadius
global aasVehicleCap
global aasCpBalance
global aasDebug
global aasL10n
global aasL10nCp
global aasL10nText

###############################################################################
#
# landot AAS mod - SETTINGS
#
#
# The CP area sizes will be calculated using the following formula:
#
# Original CP Radius * ( Max Number Of Players / aasRadius )
#
# If you set the aasRadius to 0 the CP area sizes will stay normal.
# With default setting 10:  10 players = the same, 30 players = 3 times bigger
#
aasRadius = 0
#
# Choose localization by setting the variable aasL10n
#
# 0 = English
# 1 = French
# 2 = German
#
aasL10n = 2
#
# Enable/disable Commander
#
# 1 = Enable
# 0 = Disable
#
aasCommander = 0
#
# Using the following setting you can choose whether CPs per team will
# be balanced at the start of each map. Enabling this setting means that
# the battle will always start in the middle of the map.
#
# True  = Balance number of CPs per team at map start
# False = Don't balance, use default CP ownerships
#
aasCpBalance = False
#
# No user serviceable parts below this point!
#
###############################################################################

aasDebug = 0
aasLast = [ 0, 0, 0 ]
aasNextCp = [ 0, 0, 0 ]

aasTip = " * AAS * "

aasL10nCp = { "abandonedhuts" : [ "Abandoned Huts" , "Baraquements dsaffects", "Verlassene Htten" ] ,
	      "abandonedvillage" : [ "Abandoned Village" , "Village abandonn", "Verlassenes Dorf" ] ,
	      "abandonedbunker" : [ "Abandoned Bunker" , "Abandoned Bunker", "Abandoned Bunker" ] ,
	      "airfield" : [ "Airfield" , "Arodrome", "Flugplatz" ] ,
	      "airport" : [ "Airport" , "Arodrome", "Flugplatz" ] ,
	      "airdrop" : [ "Air Drop" , "Air Drop", "Air Drop" ] ,
	      "airfieldbase" : [ "Airfield Base" , "Base de l arodrome", "Basis am Flugplatz" ] ,
	      "airfieldcamp" : [ "Airfield Camp" , "Camp de l arodrome", "Lager am Flugplatz" ] ,
	      "alley" : [ "Alley" , "Rue", "Gasse" ] ,
	      "alpha" : [ "Alpha" , "Alpha", "Alpha" ] ,
	      "alleyway" : [ "Alleyway" , "Ruelle", "Durchgang" ] ,
	      "ammodepot" : [ "Ammo Depot" , "Dpt de munitions", "Munitionslager" ] ,
	      "amibase" : [ "USMC Base" , "USMC Base", "USMC Base" ] ,
	      "backyard" : [ "Backyard" , "Arrire-cour", "Hinterhof" ] ,
	      "barracks" : [ "Barracks" , "Barracks", "Barracks" ] ,
	      "baseone" : [ "Base One" , "Base One", "Base One" ] ,
	      "basetwo" : [ "Base Two" , "Base Two", "Base Two" ] ,
	      "basethree" : [ "Base Three" , "Base Three", "Base Three" ] ,
	      "beach" : [ "Beach" , "Plage", "Strand" ] ,
	      "beachhead" : [ "Beach Head" , "Beach Head", "Beach Head" ] ,
	      "beachhouse" : [ "Beach House" , "Maison ctire", "Strandhaus" ] ,
	      "bravo" : [ "Bravo" , "Bravo", "Bravo" ] ,
	      "bridge" : [ "Bridge" , "Pont", "Brcke" ] ,
	      "bridgebase" : [ "Bridge Base" , "Base du pont", "Brckenbasis" ] ,
	      "bridgecamp" : [ "Bridge Camp" , "Camp du pont", "Brckenlager" ] ,
	      "bridgecheckpoint" : [ "Bridge Checkpoint" , "Bridge Checkpoint", "Bridge Checkpoint" ] ,
	      "bunker" : [ "Bunker" , "Bunker", "Bunker" ] ,
	      "bunkers" : [ "Bunkers" , "Bunkers", "Bunkers" ] ,
	      "boatdockstorage" : [ "Boat Dock Storage" , "Boat Dock Storage", "Boat Dock Storage" ] ,
	      "building_site" : [ "Construction Site" , "Chantier", "Baustelle" ] ,
	      "canyonguardpost" : [ "Canyon Guard Post" , "Poste de garde du dfil", "Wachposten an der Schlucht" ] ,
	      "carrier" : [ "Carrier" , "Carrier", "Carrier" ] ,
	      "carrierchina" : [ "Chinese Carrier" , "Chinese Carrier", "Chinese Carrier" ] ,
	      "carrierami" : [ "American Carrier" , "American Carrier", "American Carrier" ] ,
	      "cementfactory" : [ "Cement Factory" , "Cimenterie", "Zementfabrik" ] ,
              "center" : [ "Center" , "Center", "Center" ] ,
	      "centralfront" : [ "Central Front" , "Front central", "Zentrale Front" ] ,
	      "centralisland" : [ "Central Island" , "Ile centrale", "Zentrale Insel" ] ,
	      "checkpoint" : [ "Checkpoint" , "Checkpoint", "Checkpoint" ] ,
	      "chineseairfield" : [ "Chinese Airfield" , "Arodrome chinois", "Flugplatz (China)" ] ,
	      "chinesebase" : [ "Chinese Base" , "Base chinoise", "Basis (China)" ] ,
	      "chinesehq" : [ "Chinese HQ" , "PC chinois", "HQ (China)" ] ,
	      "city" : [ "City" , "Ville", "Stadt" ] ,
	      "city_entrance" : [ "City Entrance" , "Entre de la ville", "Stadt-Eingang" ] ,
	      "citypark" : [ "City Park" , "Parc", "Stadtpark" ] ,
	      "cityoffice" : [ "City Office" , "City Office", "City Office" ] ,
	      "clubhouse" : [ "Club House" , "Pavillon", "Klubhaus" ] ,
	      "communicationcentral" : [ "Communication Central" , "Central de communication", "Kommunikationszentrale" ] ,
	      "constructionsite" : [ "Construction Site" , "Chantier", "Baustelle" ] ,
	      "construct" : [ "Construction Site" , "Chantier", "Baustelle" ] ,
	      "construction" : [ "Construction Site" , "Chantier", "Baustelle" ] ,
	      "containerstorage" : [ "Container Storage" , "Zone de stockage", "Lagerhaus" ] ,
	      "controlcenter" : [ "Control Center" , "Centre de contrle", "Kontrollzentrum" ] ,
	      "convoy" : [ "Convoy" , "Convoi", "Konvoi" ] ,
	      "courtyard" : [ "Courtyard" , "Courtyard", "Courtyard" ] ,
	      "docks" : [ "Docks" , "Quais", "Docks" ] ,
	      "dam" : [ "Dam" , "Dam", "Dam" ] ,
	      "depot" : [ "Depot" , "Depot", "Depot" ] ,
	      "damoutpost" : [ "Dam Outpost" , "Dam Outpost", "Dam Outpost" ] ,
	      "eastoilfield" : [ "East Oilfield" , "Champ ptrolifre Est", "stliches lfeld" ] ,
	      "eastriver" : [ "East River" , "East River", "East River" ] ,
	      "estate" : [ "Estate" , "Estate", "Estate" ] ,
	      "eastisland" : [ "East Island" , "East Island", "East Island" ] ,
	      "emineentrance" : [ "East Mine Entrance" , "Entre Est de la mine", "Eingang zur stlichen Mine" ] ,
	      "factory" : [ "Factory" , "Usine", "Fabrik" ] ,
	      "factorydistrict" : [ "Factory District" , "Factory District", "Factory District" ] ,
	      "factoryintersection" : [ "Factory Intersection" , "Factory Intersection", "Factory Intersection" ] ,
	      "farm" : [ "Farm" , "Ferme", "Bauernhof" ] ,
	      "fishfactory" : [ "Fish Factory" , "Conserverie de poisson", "Fischfabrik" ] ,
	      "fortification" : [ "Fortification" , "Fortification", "Befestigungsanlage" ] ,
	      "gasstation" : [ "Gas Station" , "Station-service", "Tankstelle" ] ,
	      "gasstationfactory" : [ "Gas Station Factory" , "Gas Station Factory", "Gas Station Factory" ] ,
	      "petrol_station" : [ "Gas Station" , "Station-service", "Tankstelle" ] ,
	      "gatehouse" : [ "Gatehouse" , "Maison de gardien", "Pfrtnerhaus" ] ,
	      "heli" : [ "Heli" , "Heli", "Heli" ] ,
	      "headquarters" : [ "Headquarters" , "Headquarters", "Headquarters" ] ,
	      "hqus" : [ "USMC HQ" , "USMC HQ", "USMC HQ" ] ,
	      "hqred" : [ "HQ Red" , "HQ Red", "HQ Red" ] ,
	      "hqblue" : [ "HQ Blue" , "HQ Blue", "HQ Blue" ] ,
	      "hqmec" : [ "MEC HQ" , "MEC HQ", "MEC HQ" ] ,
	      "harbor" : [ "Harbor" , "Port", "Hafen" ] ,
	      "hill" : [ "Hill" , "Colline", "Hgel" ] ,
	      "hillvillage" : [ "Hill Village" , "Village sur la colline", "Bergdorf" ] ,
	      "hotel" : [ "Hotel" , "Htel", "Hotel" ] ,
	      "grand_hotel" : [ "Hotel" , "Htel", "Hotel" ] ,
	      "industrialarea" : [ "Industrial Area" , "Zone industrielle", "Industriegebiet" ] ,
	      "intake" : [ "Intake" , "Chemine", "Einlaufturm" ] ,
	      "island" : [ "Island" , "Ile", "Insel" ] ,
	      "islandfarmhouse" : [ "Island Farmhouse" , "Ferme insulaire", "Insel-Bauernhaus" ] ,
	      "lowerdam" : [ "Lower Dam" , "Pied du barrage", "Unterer Damm" ] ,
	      "kyongan" : [ "Kyongan" , "Kyongan", "Kyongan" ] ,
	      "mainentrance" : [ "Main Entrance" , "Entre principale", "Haupteingang" ] ,
	      "maintenancebuilding" : [ "Maintenance Buildings" , "Locaux d entretien", "Wartungsgebude" ] ,
	      "market" : [ "Market Place" , "March", "Marktplatz" ] ,
	      "marketplace" : [ "Market Place" , "Place du march", "Marktplatz" ] ,
	      "materialstation" : [ "Material Station" , "Dpt de matriel", "Materialstation" ] ,
	      "mecairfield" : [ "MEC Airfield" , "Arodrome CMO", "Flugplatz (MEC)" ] ,
	      "mecbase" : [ "MEC Base" , "Base CMO", "Basis (MEC)" ] ,
	      "chinesearmordivision" : [ "PLA Armor Division HQ" , "PLA Armor Division HQ", "PLA Armor Division HQ" ] ,
	      "MEC_Base" : [ "MEC Base" , "Base CMO", "Basis (MEC)" ] ,
              "mecdrop" : [ "MEC Drop", "MEC Drop", "MEC Drop" ] ,
	      "mosque" : [ "Mosque", "Mosque", "Mosque" ] ,
	      "mosquesquare" : [ "Mosque Square", "Mosque Square", "Mosque Square" ] ,
	      "mountainbase" : [ "Mountain Base" , "Mountain Base", "Mountain Base" ] ,
	      "mountainlookout" : [ "Mountain Lookout" , "Poste de guet", "Beobachtungsposten auf dem Berg" ] ,
	      "naturalgasstation" : [ "Natural Gas Station" , "Exploitation de gaz naturel", "Erdgas-Werk" ] ,
	      "ngasstation" : [ "North Gas Station" , "Station-service Nord", "Nrdliche Tankstelle" ] ,
              "north" : [ "North" , "North", "North" ] ,
	      "northbase" : [ "North Base" , "North Base", "North Base" ] ,
	      "northbeach" : [ "North Beach" , "Plage Nord", "Nrdlicher Strand" ] ,
	      "northdocks" : [ "North Docks" , "Quais Nord", "Nrdliche Docks" ] ,
	      "northfront" : [ "North Front" , "Front Nord", "Nrdliche Front" ] ,
	      "northisland" : [ "North Island" , "Ile septentrionale", "Nrdliche Insel" ] ,
	      "northvillage" : [ "North Village" , "North Village", "North Village" ] ,
	      "northperimeter" : [ "North Perimeter" , "Nord du primtre", "Randstellung Nord" ] ,
	      "oasis" : [ "The Oasis" , "The Oasis", "The Oasis" ] ,
	      "officebuildings" : [ "Office Buildings" , "Bureaux", "Brogebude" ] ,
	      "oilcisterns" : [ "Oil Cisterns" , "Citernes d essence", "l-Zisternen" ] ,
	      "oilfield" : [ "Oilfield" , "champ ptrol.", "lfeld" ] ,
	      "oilfieldcheckpoint" : [ "Oilfield Checkpoint" , "Point de contrle champ ptrol.", "Checkpoint lfeld" ] ,
	      "oldcity" : [ "Old City" , "Vieille ville", "Alte Stadt" ] ,
	      "olivefort" : [ "Olive Hill Fortification" , "Fortif. colline des oliviers", "Olivenberg-Befestigung" ] ,
	      "outpost" : [ "Outpost" , "Avant-poste", "Vorposten" ] ,
	      "outskirts" : [ "Outskirts" , "Faubourgs", "Auenbezirke" ] ,
	      "paraeast" : [ "Paradrop East" , "Paradrop East", "Paradrop East" ] ,
	      "parawest" : [ "Paradrop West" , "Paradrop West", "Paradrop West" ] ,
	      "petrol_station" : [ "Gas Station" , "Station-service", "Tankstelle" ] ,
	      "powerplant" : [ "Power Plant" , "Centrale lectrique", "Elektrizittswerk" ] ,
	      "powerstation" : [ "Power Station" , "Centrale lectrique", "Kraftwerk" ] ,
	      "powerstationcomm" : [ "Power Station Military Command" , "Power Station Military Command", "Power Station Military Command" ] ,
	      "reactors" : [ "Reactor Towers" , "Chemines", "Reaktoren" ] ,
	      "refinery" : [ "Refinery" , "Raffinerie", "Raffinerie" ] ,
	      "refineryentrance" : [ "Refinery Entrance" , "Entre de la raffinerie", "Eingang zur Raffinerie" ] ,
	      "refuellingstation" : [ "Refueling Station" , "Poste de ravitaillement", "Tankstation" ] ,
	      "riverfort" : [ "River Fortification" , "Fortification fluviale", "Fluss-Befestigung" ] ,
	      "rivervillage" : [ "River Village" , "Village en bord de rivire", "Dorf am Fluss" ] ,
	      "roadblock" : [ "Road Block" , "Barrage routier", "Straensperre" ] ,
	      "roadconstruct" : [ "Road Construction" , "Route en travaux", "Straenbaustelle" ] ,
	      "rockfort" : [ "Rock Fortification" , "Fortification du rocher", "Fels-Befestigung" ] ,
	      "securityhq" : [ "Security Headquarters" , "QG de la scurit", "Sicherheits-Hauptquartier" ] ,
	      "sgasstation" : [ "South Gas Station" , "Station-service Sud", "Sdliche Tankstelle" ] ,
	      "smallairstrip" : [ "Small Airstrip" , "Petite piste d aviation", "Kleines Flugfeld" ] ,
              "south" : [ "South" , "South", "South" ] ,
	      "southbase" : [ "South Base" , "South Base", "South Base" ] ,
	      "southbeach" : [ "South Beach" , "Plage Sud", "Sdlicher Strand" ] ,
	      "southblock" : [ "South Block" , "Pt de maisons Sud", "Sdlicher Block" ] ,
	      "southdocks" : [ "South Docks" , "Quais Sud", "Sdliche Docks" ] ,
	      "southfront" : [ "South Front" , "Front Sud", "Sdliche Front" ] ,
	      "southisland" : [ "South Island" , "Ile mridionale", "Sdliche Insel" ] ,
	      "southperimeter" : [ "South Perimeter" , "Sud du primtre", "Randstellung Sd" ] ,
	      "square" : [ "Square" , "Place", "Platz" ] ,
	      "the_square" : [ "Square" , "Place", "Platz" ] ,
	      "Stingers" : [ "Stingers" , "Stingers", "Stingers" ] ,
	      "suburb" : [ "Suburb" , "Banlieue", "Vorort" ] ,
	      "storagedepot" : [ "Storage Depot" , "Storage Depot", "Storage Depot" ] ,
	      "servicestation" : [ "Service Station" , "Service Station", "Service Station" ] ,
	      "supervisorbase" : [ "Supervisor Base" , "Base de surveillance", "Basis (Aufseher)" ] ,
	      "surveillancepost" : [ "Surveillance Post" , "Poste de surveillance", "Kontrollposten" ] ,
	      "temple" : [ "Temple" , "Temple", "Tempel" ] ,
	      "trainaccident" : [ "Train Accident" , "Accident ferroviaire", "Zugunglck" ] ,
	      "traindepot" : [ "Train Depot" , "Train Depot", "Train Depot" ] ,
	      "tvstation" : [ "TV Station" , "Station TV", "Fernsehsender" ] ,
	      "tvbalcony" : [ "TV Balcony" , "TV Balcony", "TV Balcony" ] ,
	      "uppercamp" : [ "Upper Camp" , "Camp suprieur", "Oberes Lager" ] ,
	      "upperdam" : [ "Upper Dam" , "Sommet du barrage", "Oberer Damm" ] ,
	      "usairfield" : [ "US Airfield" , "Arodrome amricain", "Flugplatz (US)" ] ,
	      "usair" : [ "USMC Air Base" , "Arodrome amricain", "Flugplatz (US)" ] ,
	      "uscarrier" : [ "US Carrier" , "Porte-avions amricain", "Flugzeugtrger (US)" ] ,
              "usdrop" : [ "US Drop" , "US Drop", "US Drop" ] ,
	      "ustown" : [ "US Town" , "Ville amricaine", "Stadt (US)" ] ,
	      "usbase" : [ "USMC Base" , "USMC Base", "USMC Base" ] ,
	      "usassaultforce" : [ "USMC Assault Force" , "USMC Assault Force", "USMC Assault Force" ] ,
	      "usassaultcamp" : [ "USMC Assault Camp" , "USMC Assault Camp", "USMC Assault Camp" ] ,
	      "village" : [ "Village" , "Village", "Verlassenes Dorf" ] ,
	      "villagefort" : [ "Village Fortification" , "Fortification du village", "Dorf-Befestigung" ] ,
	      "vistapoint" : [ "Vista Point" , "Point de vue", "Aussichtspunkt" ] ,
	      "warehouse" : [ "Warehouse" , "Entrept", "Lagerhaus" ] ,
	      "waterfall" : [ "Dam" , "Barrage", "Damm" ] ,
	      "waterfront" : [ "Waterfront" , "Waterfront", "Waterfront" ] ,
	      "westisland" : [ "West Island" , "West Island", "West Island" ] ,
	      "westoilfield" : [ "West Oilfield" , "Champ ptrolifre Ouest", "Westliches lfeld" ] ,
	      "westperimeter" : [ "West Perimeter" , "Ouest du primtre", "Randstellung West" ] ,
	      "westriver" : [ "West River" , "West River", "West River" ] ,
	      "wmineentrance" : [ "West Mine Entrance" , "Entre Ouest de la mine", "Eingang zur westlichen Mine" ] ,
	      "woodyard" : [ "Wood Yard" , "Parc  bois", "Holzlagerplatz" ] }

aasL10nTexts = [ [ "Flags must be captured in correct order!", "You have entered the ", " must be captured before the ", "Proceed to the " ] ,
		 [ "Capturez les points dans le bon ordre!", "Vous tes au point : ", " : doit tre captur avant : ", "Capturez : " ] ,
		 [ "Die Flaggen mssen in einer bestimmten Reihenfolge erobert werden!", "Gerade erobert: ", " muss zuerst erobert werden vor ", "Nchster Eroberungspunkt: " ] ]

aasL10nText = aasL10nTexts[aasL10n]

def aasSetNextCp(aasTeam, aasInit):
	
	aasMaps = { "16" :
		    { "sf_karkand" :
		      [ "cpname_sk_16_market", "cpname_sk_16_square", "cpname_sk_16_hotel", "cpname_sk_16_gasstation" ] } ,
		    "32" :
		    { "sf_karkand" :
                      [ "cpname_sk_32_gatehouse", "cpname_sk_32_trainaccident", "cpname_sk_32_market", "cpname_sk_32_square", "cpname_sk_32_hotel", "cpname_sk_32_gasstation" ] ,
                      "wetland_assault" :
		      [ "cpname_zw_32_farm", "cpname_zw_32_naturalgasstation", "cpname_zw_32_centralisland", "cpname_zw_32_village" ] } ,
		    "64" :
		    { "dragon_valley_xtreme":
                      [ "cpname_dv_64_refinery", "cpname_dv_64_powerstation", "cpname_dv_64_woodyard", "cpname_dv_64_islandfarmhouse", "cpname_dv_64_vistapoint", "cpname_dv_64_uscarrier" ] ,
                      "fushe_pass_canyon_flight":
                      [ "cpname_fp_64_chairfield", "cpname_fp_64_industry", "cpname_fp_64_factory", "cpname_fp_64_usairfield" ] ,
                      "karkand_bridge_fight":
                      [ "cpname_kb_64_mecbase", "cpname_kb_64_gatehouse", "cpname_kb_64_trainaccident", "cpname_kb_64_usbase" ] ,
                      "kubra_dam_bridge_fight":
                      [ "cpname_kd_64_mecdrop", "cpname_kd_64_north", "cpname_kd_64_center", "cpname_kd_64_south", "cpname_kd_64_usdrop" ] ,
                      "sf_karkand" :
		      [ "cpname_sk_64_factory", "cpname_sk_64_warehouse", "cpname_sk_64_cementfactory", "cpname_sk_64_gatehouse", "cpname_sk_64_trainaccident", "cpname_sk_64_suburb", "cpname_sk_64_square", "cpname_sk_64_hotel", "cpname_sk_64_gasstation" ] ,
		      "wake_island_2007" :
		      [ "cpname_wi_64_southbase", "cpname_wi_64_beach", "cpname_wi_64_airfield", "cpname_wi_64_northbase", "cpname_wi_64_northvillage", "cpname_wi_64_carrier" ] } }


	aasCps = False
	for obj in bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint'):
		obj = obj.templateName
		aasMapSize = obj[10:12]
		break
	aasMapsOfSize = aasMaps[aasMapSize]
	aasCps = aasMapsOfSize[bf2.gameLogic.getMapName()]

	aasTeam2 = 2
	if aasCps:
		if aasTeam == 2:
			aasTeam2 = 1
			aasCps.reverse()
		for aasCnt in range(0,len(aasCps)):
			aasNumOfCps = 0
			aasTeamCps = 0
			for obj in bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint'):
				aasNumOfCps += 1
				if obj.cp_getParam('team') == aasTeam:
					aasTeamCps += 1
				if obj.templateName.lower() == aasCps[aasCnt]:
					aasCpId = obj
				elif obj.templateName.lower() == aasCps[aasCnt-1]:
					aasPrevCpId = obj
			if aasInit:
				if aasTeamCps < int(aasNumOfCps / 2):
					aasCpId.cp_setParam('flag', aasTeam)
					aasCpId.cp_setParam('team', aasTeam)
					aasCpId.cp_setParam('takeOverChangePerSecond', 100)
			elif aasCpId.cp_getParam('team') != aasTeam:
				if aasCnt == 0 or aasPrevCpId.cp_getParam('team') == aasTeam:
					aasNextCp[aasTeam] = aasCps[aasCnt]
					break
					
def aasCpName(aasCp):
	aasCp = aasCp.lower()
	aasCp = aasL10nCp[aasCp[13:]]
	aasCp = aasCp[aasL10n]
	return aasCp


def init():
	# events hook
	host.registerGameStatusHandler(onGameStatusChanged)
	if host.sgl_getIsAIGame() == 1:
		host.sh_setEnableCommander(aasCommander)
	else:
		host.sh_setEnableCommander(aasCommander)
		
	host.registerHandler('TimeLimitReached', onTimeLimitReached, 1)	

	if g_debug: print "gpm_cq.py initialized"
		
		
		
def deinit():
	bf2.triggerManager.destroyAllTriggers()
	global g_controlPoints
	g_controlPoints = []
	host.unregisterGameStatusHandler(onGameStatusChanged)
	if g_debug: print "gpm_cq.py uninitialized"



def onGameStatusChanged(status):

	global g_controlPoints
	if status == bf2.GameStatus.Playing: 

		if aasCpBalance:
			aasSetNextCp(1, 1)
			aasSetNextCp(2, 1)

		# add control point triggers
		g_controlPoints = bf2.objectManager.getObjectsOfType('dice.hfe.world.ObjectTemplate.ControlPoint')
		for obj in g_controlPoints:
			radius = float(obj.getTemplateProperty('radius'))
			isHemi = int(obj.cp_getParam('isHemisphere'))

			if aasRadius > 0:
				aasRadiusFactor = float(host.ss_getParam('maxPlayers')) / aasRadius
				if aasRadiusFactor > 1:
					radius = round(radius * aasRadiusFactor)
			
			if isHemi != 0:
				id = bf2.triggerManager.createHemiSphericalTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))
			else:
				id = bf2.triggerManager.createRadiusTrigger(obj, onCPTrigger, '<<PCO>>', radius, (1, 2, 3))			
			obj.triggerId = id
			obj.lastAttackingTeam = 0
			if obj.cp_getParam('team') > 0:
				obj.flagPosition = Top
			else:
				obj.flagPosition = Bottom

		host.registerHandler('ControlPointChangedOwner', onCPStatusChange)

		# setup ticket system
		ticketsTeam1 = calcStartTickets(bf2.gameLogic.getDefaultTickets(1))
		ticketsTeam2 = calcStartTickets(bf2.gameLogic.getDefaultTickets(2))
		
		bf2.gameLogic.setTickets(1, ticketsTeam1)
		bf2.gameLogic.setTickets(2, ticketsTeam2)

		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)

		bf2.gameLogic.setTicketLimit(1, 1, 0)
		bf2.gameLogic.setTicketLimit(2, 1, 0)
		bf2.gameLogic.setTicketLimit(1, 2, 10)
		bf2.gameLogic.setTicketLimit(2, 2, 10)
		bf2.gameLogic.setTicketLimit(1, 3, int(ticketsTeam1*0.1))
		bf2.gameLogic.setTicketLimit(2, 3, int(ticketsTeam2*0.1))
		bf2.gameLogic.setTicketLimit(1, 4, int(ticketsTeam1*0.2))
		bf2.gameLogic.setTicketLimit(2, 4, int(ticketsTeam1*0.2))
		
		host.registerHandler('TicketLimitReached', onTicketLimitReached)
		updateTicketLoss()

		# player events
		host.registerHandler('PlayerDeath', onPlayerDeathCQ)
		host.registerHandler('PlayerKilled', onPlayerKilledCQ)
		host.registerHandler('PlayerRevived', onPlayerRevived)
		host.registerHandler('PlayerSpawn', onPlayerSpawn)
		host.registerHandler('EnterVehicle', onEnterVehicle)
		host.registerHandler('ExitVehicle', onExitVehicle)

		if g_debug: print "Conquest gamemode initialized."
	else:
		bf2.triggerManager.destroyAllTriggers()
		g_controlPoints = []



def calcStartTickets(mapDefaultTickets):
	return int(mapDefaultTickets * (bf2.serverSettings.getTicketRatio() / 100.0))
	
	
	
def onTimeLimitReached(value):
	team1tickets = bf2.gameLogic.getTickets(1)
	team2tickets = bf2.gameLogic.getTickets(2)
	
	winner = 0
	victoryType = 0
	if team1tickets > team2tickets:
		winner = 1
		victoryType = 3
	elif team2tickets > team1tickets:
		winner = 2
		victoryType = 3
	

	host.sgl_endGame(winner, victoryType)



# update ticket system
def updateTicketLoss():
	areaValueTeam1 = 0
	areaValueTeam2 = 0
	totalAreaValue = 0
	numCpsTeam0 = 0
	numCpsTeam1 = 0
	numCpsTeam2 = 0
	
	# calculate control point area value for each team
	for obj in g_controlPoints:
		team = obj.cp_getParam('team')
		if team == 1:
			areaValueTeam1 += obj.cp_getParam('areaValue', team)
			totalAreaValue += areaValueTeam1
			numCpsTeam1 += 1
		elif team == 2:
			areaValueTeam2 += obj.cp_getParam('areaValue', team)
			totalAreaValue += areaValueTeam2
			numCpsTeam2 += 1
		else:
			numCpsTeam0 += 1
			totalAreaValue += 0
		
	# check if a team has no control points
	if numCpsTeam1 == 0 or numCpsTeam2 == 0:
		if numCpsTeam1 == 0:
			losingTeam = 1
			winningTeam = 2
		else:
			losingTeam = 2
			winningTeam = 1
		
		# check if there is anyone alive
		foundLivingPlayer = False
		for p in bf2.playerManager.getPlayers():
			if p.getTeam() == losingTeam and p.isAlive():
				foundLivingPlayer = True
				break
				
		if not foundLivingPlayer:

			# drop tickets
			ticketLossPerSecond = bf2.gameLogic.getDefaultTicketLossAtEndPerMin()
			bf2.gameLogic.setTicketChangePerSecond(losingTeam, -ticketLossPerSecond)
			bf2.gameLogic.setTicketChangePerSecond(winningTeam, 0)
			
			return

	
	# update ticket loss
	team1AreaOverweight = areaValueTeam1 - areaValueTeam2
	percentualOverweight = 1.0
	if totalAreaValue != 0:
		percentualOverweight = abs(team1AreaOverweight / totalAreaValue)
	
	ticketLossPerSecTeam1 = calcTicketLossForTeam(1, areaValueTeam2, -team1AreaOverweight)
	ticketLossPerSecTeam2 = calcTicketLossForTeam(2, areaValueTeam1, team1AreaOverweight)
	bf2.gameLogic.setTicketChangePerSecond(1, -ticketLossPerSecTeam1)
	bf2.gameLogic.setTicketChangePerSecond(2, -ticketLossPerSecTeam2)


	
# actual ticket loss calculation function
def calcTicketLossForTeam(team, otherTeamAreaValue, otherTeamAreaOverweight):
	if otherTeamAreaValue >= 100 and otherTeamAreaOverweight > 0:
		ticketLossPerSecond = (bf2.gameLogic.getDefaultTicketLossPerMin(team) / 60.0) * (otherTeamAreaOverweight / 100.0)
		return ticketLossPerSecond
	else:
		return 0


	
DOWNWARDS = -1
UPWARDS = 1	

# called when tickets reach a predetermined limit (negativ value means that the tickets have become less than the limit)
def onTicketLimitReached(team, limitId):
	if (limitId == -1):
		if (team == 1):
			winner = 2
		
		elif (team == 2):
			winner = 1
		
		bf2.gameLogic.setTicketState(1, 0)
		bf2.gameLogic.setTicketState(2, 0)
	
		host.sgl_endGame(winner, 3)		

	# update ticket state
	else:
		updateTicketWarning(team, limitId)

	

# called when the ticket state should be updated (for triggering messages and sounds based on tickets left)
def updateTicketWarning(team, limitId):

	oldTicketState = bf2.gameLogic.getTicketState(team)
	newTicketState = 0
	
	if (oldTicketState >= 10):
		newTicketState = 10		

	if (limitId == -2):
		newTicketState = 10
	
	elif (limitId == 2):
		newTicketState = 0		

	elif (limitId == -3):
		newTicketState += 2

	elif (limitId == -4):
		newTicketState += 1

	if (oldTicketState != newTicketState):
		bf2.gameLogic.setTicketState(team, newTicketState)
		
	
	
# called when someone enters or exits cp radius
def onCPTrigger(triggerId, cp, vehicle, enter, userData):
	if not cp.isValid(): return
	
	if vehicle and vehicle.getParent(): return
	
	# can this cp be captured at all?
	if cp.cp_getParam('unableToChangeTeam') != 0:
		return					
		
	playersInVehicle = None	
	if vehicle:
		playersInVehicle = vehicle.getOccupyingPlayers()
	
	if enter:
		for p in playersInVehicle:
			cp = getOccupyingCP(p)
			if cp != None:
				if not p.getIsInsideCP():
					if g_debug: print "Resetting enterPctAt for player ", p.getName()
					p.enterCpAt = host.timer_getWallTime()
	
	if vehicle:	
		for p in playersInVehicle:
			# only count first player in a vehicle
			if p == playersInVehicle[0]:  
				print p.index, " is in radius. v=", vehicle.templateName
				p.setIsInsideCP(enter)	
			else:
				p.setIsInsideCP(0)
				if enter == 1:
					bf2.gameLogic.sendHudEvent(p, 66, 49) #66 = HEEnableHelpMessage, 49 = VHMExitToCaptureFlag;			

	# count people in radius
	team1Occupants = 0
	team2Occupants = 0

	pcos = bf2.triggerManager.getObjects(cp.triggerId)
	for o in pcos:
		if not o: continue # you can get None in the result tuple when the host can't figure out what object left the trigger
		if o.getParent(): continue # getOccupyingPlayers returns all players downwards in the hierarchy, so dont count them twice
		occupyingPlayers = o.getOccupyingPlayers()
		for p in occupyingPlayers:
			
			# only count first player in a vehicle
			if p != occupyingPlayers[0]: 
				continue
				
			if p.isAlive() and not p.isManDown():
				
				if not p.killed:
					if p.getTeam() == 1:
						team1Occupants += 1
					elif p.getTeam() == 2:
						team2Occupants += 1


	# determine who is taking control
	team1OverWeight = team1Occupants - team2Occupants
	attackOverWeight = 0

	if team1OverWeight > 0:
		attackingTeam = 1
	elif team1OverWeight < 0:
		attackingTeam = 2
	else:
		attackingTeam = 0

	
	if team1Occupants == 0 and team2Occupants == 0:

		# nobody here, slowly go back to owning team
		if cp.cp_getParam('team') == 0:
			attackOverWeight = -0.5
		else:
			attackOverWeight = 0.5
			
		timeToChangeControl = cp.cp_getParam('timeToLoseControl')

	else:

		aasTeam = attackingTeam

		aasSetNextCp(aasTeam, 0)

		if cp.cp_getParam('team') != aasTeam:
			if aasNextCp[aasTeam] != cp.templateName.lower():
				aasMsg = 'game.sayTeam ' + str(aasTeam) + ' "' + aasTip + aasCpName(aasNextCp[aasTeam]) + aasL10nText[2] + aasCpName(cp.templateName) + aasTip + '"'
				if aasMsg != aasLast[aasTeam]:
					aasLast[aasTeam] = aasMsg
					host.rcon_invoke(aasMsg)
				return

		# raise flag if already ours, or at bottom and neutral. Otherwise lower first.
		if cp.cp_getParam('flag') == attackingTeam or (cp.flagPosition == Bottom and cp.cp_getParam('team') == 0):

			# our flag, raise
			attackOverWeight = abs(team1OverWeight)
			timeToChangeControl = cp.cp_getParam('timeToGetControl')
		else:

			# other team raised flag, lower first
			attackOverWeight = - abs(team1OverWeight)
			timeToChangeControl = cp.cp_getParam('timeToLoseControl')


	if cp.cp_getParam('onlyTakeableByTeam') != 0 and cp.cp_getParam('onlyTakeableByTeam') != attackingTeam:
		return


	# flag can only be changed when at bottom
	if cp.flagPosition == Bottom:
		cp.cp_setParam('flag', attackingTeam)


	# calculate flag raising/lowering speed
	if timeToChangeControl > 0:
		takeOverChangePerSecond = 1.0 * attackOverWeight / timeToChangeControl
	else:
		takeOverChangePerSecond = 0.0

	if (cp.flagPosition == Top and takeOverChangePerSecond > 0) or (cp.flagPosition == Bottom and takeOverChangePerSecond < 0):
		takeOverChangePerSecond = 0.0

	if abs(takeOverChangePerSecond) > 0:
		cp.flagPosition = Middle
				
	cp.cp_setParam('takeOverChangePerSecond', takeOverChangePerSecond)

			
	
# called when a control point flag reached top or bottom
def onCPStatusChange(cp, top):

	playerId = -1
	takeoverType = -1
	newTeam = -1
	scoringTeam = -1
	
	if top:	cp.flagPosition = Top
	else:   cp.flagPosition = Bottom
	
	# determine capture / neutralize / defend
	if cp.cp_getParam('team') != 0:

		if top:
			# regained flag, do nothing
			pass
			
		else:
			# neutralize
			newTeam = 0
			if cp.cp_getParam('team') == 1:
				scoringTeam = 2
			else:
				scoringTeam = 1
				
			takeoverType = TAKEOVERTYPE_NEUTRALIZE

	else:

		if top:
			# capture
			newTeam = cp.cp_getParam('flag')
			scoringTeam = newTeam
			takeoverType = TAKEOVERTYPE_CAPTURE

		else:
			# hit bottom, but still neutral
			pass

	
	# scoring
	if takeoverType > 0:
		pcos = bf2.triggerManager.getObjects(cp.triggerId)
	
		# count number of players
		scoringPlayers = []
		firstPlayers = []
		for o in pcos:
			if o.getParent(): continue

			occupyingPlayers = o.getOccupyingPlayers()
			for p in occupyingPlayers:
			
				# only count first player in a vehicle
				if p != occupyingPlayers[0]: 
					continue
					
				if p.isAlive() and not p.isManDown() and p.getTeam() == scoringTeam:
					if len(firstPlayers) == 0 or p.enterCpAt < firstPlayers[0].enterCpAt:
						firstPlayers = [p]
					elif p.enterCpAt == firstPlayers[0].enterCpAt:
						firstPlayers += [p]
					
					if not p in scoringPlayers:
						scoringPlayers += [p]
	
		# deal score
		for p in scoringPlayers:
			oldScore = p.score.score;
			if takeoverType == TAKEOVERTYPE_CAPTURE:
				if p in firstPlayers:
					p.score.cpCaptures += 1
					addScore(p, SCORE_CAPTURE, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 0) #12 = Conquest, 0 = Capture
					playerId = p.index
				else:
					p.score.cpAssists += 1
					addScore(p, SCORE_CAPTUREASSIST, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 2) #12 = Conquest, 2 = Assist


			elif takeoverType == TAKEOVERTYPE_NEUTRALIZE:
				if p in firstPlayers:
					p.score.cpNeutralizes += 1
					addScore(p, SCORE_NEUTRALIZE, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 3) #12 = Conquest, 3 = Neutralize
				else:
					p.score.cpNeutralizeAssists += 1
					addScore(p, SCORE_NEUTRALIZEASSIST, RPL)
					bf2.gameLogic.sendGameEvent(p, 12, 4) #12 = Conquest, 4 = Neutralize assist
					
				

	# immediate ticket loss for opposite team
	enemyTicketLossInstant = cp.cp_getParam('enemyTicketLossWhenCaptured')
	if enemyTicketLossInstant > 0 and newTeam > 0:
		
		if newTeam == 1:
			punishedTeam = 2
		elif newTeam == 2:
			punishedTeam = 1
		
		tickets = bf2.gameLogic.getTickets(punishedTeam)
		tickets -= enemyTicketLossInstant
		bf2.gameLogic.setTickets(punishedTeam, tickets)
	
	
	# update control point	
	cp.cp_setParam('playerId', playerId) #always set player first
	if newTeam != -1 and cp.cp_getParam('team') != newTeam:
		cp.cp_setParam('team', newTeam)
	onCPTrigger(cp.triggerId, cp, 0, 0, 0)
	updateTicketLoss()

	for aasTeam in [ 1, 2 ]:
		aasSetNextCp(aasTeam, 0)
		aasMsg = 'game.sayTeam ' + str(aasTeam) + ' "' + aasTip + aasL10nText[3] + aasCpName(aasNextCp[aasTeam]) + aasTip + '"'
		if aasMsg != aasLast[aasTeam]:
			aasLast[aasTeam] = aasMsg
			host.rcon_invoke(aasMsg)
				
				
def onPlayerDeathCQ(victim, vehicle):		

	# punish team with one ticket
	if victim != None:
		team = victim.getTeam()
		teamTickets = bf2.gameLogic.getTickets(team)
		teamTickets -= 1
		bf2.gameLogic.setTickets(team, teamTickets)

	# check if it was the last player
	foundLivingPlayer = False
	for p in bf2.playerManager.getPlayers():
		if p != victim and p.getTeam() == victim.getTeam() and p.isAlive():
			foundLivingPlayer = True
	
	if not foundLivingPlayer:
		updateTicketLoss()
	


def onPlayerKilledCQ(victim, attacker, weapon, assists, object):
	if not victim: 
		return
	
	victim.killed = True
	
	# update flag takeover status if victim was in a CP radius
	cp = getOccupyingCP(victim)
	if cp != None:
		onCPTrigger(-1, cp, victim.getVehicle(), False, None)

		# give defend score if killing enemy within cp radius
		if attacker != None and attacker.getTeam() != victim.getTeam()\
		   and cp.cp_getParam('unableToChangeTeam') == 0 and cp.cp_getParam('onlyTakeableByTeam') == 0:
		
			if cp != None and cp.cp_getParam('team') == attacker.getTeam():
				attacker.score.cpDefends += 1
				addScore(attacker, SCORE_DEFEND, RPL)
				bf2.gameLogic.sendGameEvent(attacker, 12, 1) #12 = Conquest, 1 = Defend
			
			
			
def onPlayerRevived(victim, attacker):

	# update flag takeover status if victim was in a CP radius
	victim.killed = False
	
	# Project Reality Mod Revive Player at 50 Health
	victim.getDefaultVehicle().setDamage(50)
	
	cp = getOccupyingCP(victim)
	if cp != None:
		onCPTrigger(-1, cp, victim.getVehicle(), True, None)
	
	
	
def onPlayerSpawn(player, soldier):
	player.killed = False
	
	aasTeam = player.getTeam()

	aasMsg = 'game.sayTeam ' + str(aasTeam) + ' "' + aasTip + aasL10nText[0] + aasTip + '"'
	if aasMsg != aasLast[aasTeam]:
		aasLast[aasTeam] = aasMsg
		host.rcon_invoke(aasMsg)

	aasSetNextCp(aasTeam, 0)

	aasMsg = 'game.sayTeam ' + str(aasTeam) + ' "' + aasTip + aasL10nText[3] + aasCpName(aasNextCp[aasTeam]) + aasTip + '"'
	if aasMsg != aasLast[aasTeam]:
		aasLast[aasTeam] = aasMsg
		host.rcon_invoke(aasMsg)
	
def onEnterVehicle(player, vehicle, freeSoldier = False):
	player.setIsInsideCP(False)
	cp = getOccupyingCP(player)
	if cp != None:
		onCPTrigger(-1, cp, vehicle, False, None)
		updateCaptureStatus(vehicle) 


def onExitVehicle(player, vehicle):

	# update flag takeover status if player in a CP radius
	
	print "Player exiting ", player.getName()
	cp = getOccupyingCP(player)

	# can this cp be captured at all?
	player.setIsInsideCP(cp != None and cp.cp_getParam('unableToChangeTeam') == 0)

	updateCaptureStatus(vehicle)
		

#Update cp capture status on players in vehicle
def updateCaptureStatus(vehicle):	

	rootVehicle = bf2.objectManager.getRootParent(vehicle)
	playersInVehicle = rootVehicle.getOccupyingPlayers()
	
	# set the player in the topmost pco as inside - others outside
	for p in playersInVehicle:
		print "Players in vehicle ", p.getName()
		cp = getOccupyingCP(p)
		p.setIsInsideCP(cp != None and cp.cp_getParam('unableToChangeTeam') == 0 and p == playersInVehicle[0])
		
		
# find cp that player is occupying, if any		
def getOccupyingCP(player):
	vehicle = player.getVehicle()
	playerPos = vehicle.getPosition()
	
	# find closest CP
	closestCP = None
	if len(g_controlPoints) == 0: return None
	for obj in g_controlPoints:
		distanceTo = getVectorDistance(playerPos, obj.getPosition())
		if closestCP == None or distanceTo < closestCPdist:
			closestCP = obj
			closestCPdist = distanceTo
	
	# is the player in radius?
	pcos = bf2.triggerManager.getObjects(closestCP.triggerId)
	for o in pcos:
		if o == player.getDefaultVehicle():
			# Player is DEFAULT vehicle - this is needed when called from onEnterVehicle
			return closestCP
		else:
			for p in o.getOccupyingPlayers():
				if p == player:
					return closestCP
	
	return None



# get distance between two positions
def getVectorDistance(pos1, pos2):
	diffVec = [0.0, 0.0, 0.0]
	diffVec[0] = math.fabs(pos1[0] - pos2[0])
	diffVec[1] = math.fabs(pos1[1] - pos2[1])
	diffVec[2] = math.fabs(pos1[2] - pos2[2])
	 
	return math.sqrt(diffVec[0] * diffVec[0] + diffVec[1] * diffVec[1] + diffVec[2] * diffVec[2])
		
		
	
